#include <iostream>
#include <string>
#include <vector>

#include <kylin-ai/business/data-management/datamanagement.h>

bool checkStatus(DataManagementStatus status)
{
  if (!status.success)
  {
    std::cerr << "Get error: " << status.errorMsg << std::endl;
    return false;
  }
  return true;
}

void test01_test_files()
{
  std::cout << "Start test01_test_files" << std::endl;
  std::cout << "===========================================" << std::endl;

  DataManagementSession *session = nullptr;
  session = data_management_create_session();
  if (session == nullptr)
  {
    std::cout << "session is nullptr" << std::endl;
    return;
  }

  // 文件路径换成自己本地的路径
  std::string input_add_text_files = R"(
    [
      {
        "filepath": "/home/kylin/prj-v11/公司报销制度及流程.docx",
        "fileformat": "docx"
      }
    ])";

  // 检查文件格式是否支持

  std::vector<std::string> inputs = {"/home/kylin/prj-v11/公司报销制度及流程.docx"};

  char *result = nullptr;
  std::cout << "Start check_file_format" << std::endl;
  for (const auto &input : inputs)
  {
    DataManagementStatus status = data_management_check_file_format(session, input.c_str(), &result);
    if (checkStatus(status))
    {
      std::cout << "check_file_format Sucess : " << result << std::endl;
    }
  }

  // 添加文件
  std::cout << "Start add_text_files" << std::endl;
  DataManagementStatus status = data_management_add_text_files(session, input_add_text_files.c_str());
  if (checkStatus(status))
  {
    std::cout << "add_text_files Sucess" << std::endl;
  }

  std::string input_add_image_files = R"(
    [
      {
        "filepath": "/home/kylin/prj-v11/aiassistant_20240710_173905.png",
        "fileformat": "png"
      }
    ])";

  std::cout << "Start add_image_files" << std::endl;
  data_management_add_image_files(session, input_add_image_files.c_str());
  if (checkStatus(status))
  {
    std::cout << "add_image_files Sucess" << std::endl;
  }

  // 添加文件成功，可以进行相似搜索，如，搜索报销文本内容进行搜索，使用默认阈值
  std::cout << "Start similarity_search" << std::endl;
  std::string searchInput = R"(
    {
        "text": "报销"
    })";
  status = data_management_similarity_search(session, searchInput.c_str(), &result);
  if (checkStatus(status))
  {
    std::cout << "similarity_search Sucess" << result << std::endl;
  }

  // data_management_update_files_name,目前上游暂不支持，所以此接口暂不实现
  std::string input_update_files_name = R"(
    [
      {
      "old_filepath": "/home/kylin/操作系统大会演示素材v0.2/智能数据管理素材/文档素材/公司报销制度及流程.docx",
      "new_filepath": "/home/kylin/操作系统大会演示素材v0.2/智能数据管理素材/文档素材/公司报销制度及流程(1).docx"
      }
    ])";
  std::cout << "Start update_files_name" << std::endl;
  status = data_management_update_files_name(session, input_update_files_name.c_str());
  if (checkStatus(status))
  {
    std::cout << "update_files_name Success: " << std::endl;
  }

  // 如果文件内容有变化，需要调用data_management_update_files_content
  std::string input_update_file_content = R"(
    [
      {
        "filepath": "/home/kylin/prj-v11/公司报销制度及流程.docx",
        "fileformat": "docx"
      }
    ])";

  std::cout << "Start update_files_content" << std::endl;
  status = data_management_update_files_content(session, input_update_file_content.c_str());
  if (checkStatus(status))
  {
    std::cout << "update_files_content Success: " << std::endl;
  }

  std::string input_extract_search_conditions = R"({
        "description": "搜索一下狗狗的图片"
    })";
  std::cout << "Testing extract_search_conditions..." << std::endl;
  status = data_management_extract_search_conditions(session, input_extract_search_conditions.c_str(), &result);
  if (checkStatus(status))
  {
    std::cout << "extract_search_conditions Success:" << result << std::endl;
  }

  bool isRelevant;
  // 判断文件和内容是否相关
  std::string input_is_file_relevant_to_description = R"({
        "file": "/home/kylin/prj-v11/公司报销制度及流程.docx",
        "description": "报销"
    })";
  std::cout << "Start is_file_relevant_to_description" << std::endl;
  status = data_management_is_file_relevant_to_description(session, input_is_file_relevant_to_description.c_str(), &isRelevant);

  if (checkStatus(status))
  {
    std::cout << "is_file_relevant_to_description Success:" << isRelevant << std::endl;
  }

  std::cout << "Start get_summarys_of_files..." << std::endl;
  std::string input_get_summarys_of_files = R"({
        "files": [
            "/home/kylin/prj-v11/公司报销制度及流程.docx"
        ]
    })";
  status = data_management_get_summarys_of_files(session, input_get_summarys_of_files.c_str(), &result);
  if (checkStatus(status))
  {
    std::cout << "get_summarys_of_files Success:" << result << std::endl;
  }

  std::cout << "Start delete_files" << std::endl;
  std::string input_delete_files = R"(
    [
      {
        "filepath": "/home/kylin/prj-v11/公司报销制度及流程.docx"
      }
    ])";
  data_management_delete_files(session, input_delete_files.c_str());

  if (checkStatus(status))
  {
    std::cout << "delete_files Sucess" << std::endl;
  }

  data_management_destroy_result(result);
  data_management_destroy_session(session);

  std::cout << std::endl;
  std::cout << "===========================================" << std::endl;
}

void test02_test_tags()
{
  std::cout << "Start test02_test_tags" << std::endl;
  std::cout << "===========================================" << std::endl;

  DataManagementSession *session = data_management_create_session();
  if (session == nullptr)
  {
    std::cout << "session is nullptr" << std::endl;
    return;
  }

  char *result = nullptr;
  std::string input_get_tag_of_file = R"(
      {
        "files": ["/home/kylin/prj-v11/公司报销制度及流程.docx"]
      })";

  std::cout << "Start get_tags_of_files" << std::endl;
  DataManagementStatus status = data_management_get_tags_of_files(session, input_get_tag_of_file.c_str(), &result);
  if (checkStatus(status))
  {
    std::cout << "get_tags_of_files Success:" << result << std::endl;
  }

  std::cout << "Start get_all_tags" << std::endl;
  status = data_management_get_all_tags(session, &result);
  if (checkStatus(status))
  {
    std::cout << "get_all_tags Success:" << result << std::endl;
  }

  // 标签 相似搜索
  std::string input_search = R"(
      {
        "tags": ["户外团建活动游戏(1).pptx"]
      })";
  std::cout << "Start search_similar_tags" << std::endl;
  status = data_management_search_similar_tags(session, input_search.c_str(), &result);

  if (checkStatus(status))
  {
    std::cout << "search_similar_tags Success:" << result << std::endl;
  }

  // 根据标签 搜索文件
  std::string input_search_file = R"(
      {
        "tags": ["户外团建活动游戏(1).pptx"]
      })";
  std::cout << "Start search_files_about_tags" << std::endl;
  status = data_management_search_files_about_tags(session, input_search_file.c_str(), &result);
  if (checkStatus(status))
  {
    std::cout << "search_files_about_tags Success:" << result << std::endl;
  }

  // 判断文件和标签是否相关
  std::string is_file_relevant_to_tags = R"(
      {
        "file": "/home/kylin/prj-v11/公司报销制度及流程.docx",
        "tags": ["户外团建活动游戏(1).pptx"]
      })";
  bool isRelevant;
  std::cout << "Start is_file_relevant_to_tags" << std::endl;
  status = data_management_is_file_relevant_to_tags(session, is_file_relevant_to_tags.c_str(), &isRelevant);
  if (checkStatus(status))
  {
    std::cout << "is_file_relevant_to_tags Success:" << isRelevant << std::endl;
  }

  data_management_destroy_result(result);
  data_management_destroy_session(session);

  std::cout << std::endl;
  std::cout << "===========================================" << std::endl;
}

void testGetFeatureStatus()
{
  /*
  特征状态如下：
  typedef enum { UNKNOWN = -1, AVAILABLE = 0, NOT_INSTALLED, NOT_SUPPORTED } FeatureStatus;
  */

  DataManagementSession *session = data_management_create_session();
  FeatureStatus status;
  std::cout << "Start get_feature_status" << std::endl;
  DataManagementStatus ret = data_management_get_feature_status(session, &status);
  if (checkStatus(ret))
  {
    std::cout << "get_feature_status Sucess : " << status << std::endl;
  }

  data_management_destroy_session(session);
  std::cout << std::endl;
}
void testGetAllFileInfo()
{
  DataManagementSession *session = data_management_create_session();
  char *result = nullptr;
  std::cout << "Start get_all_fileinfos" << std::endl;
  DataManagementStatus status = data_management_get_all_fileinfos(session, &result);
  if (checkStatus(status))
  {
    std::cout << "get_all_fileinfos Sucess : " << result << std::endl;
  }

  data_management_destroy_result(result);
  data_management_destroy_session(session);
  std::cout << std::endl;
}

int main()
{
  /*
  测试所用的文件路径需要换成自己电脑本地的路径
  */

  testGetFeatureStatus();
  test01_test_files();
  testGetAllFileInfo();
  test02_test_tags();

  return 0;
}